@@ -1,5 +1,5 @@
 """Write Command."""
-from .common import add_msgdef_args, create_ebus, disable_stdout_buffering, load_msgdefs
+from .common import add_msgdef_args, async_load_msgdefs, create_ebus, disable_stdout_buffering
 
 
 def parse_args(subparsers):
@@ -14,7 +14,7 @@ def parse_args(subparsers):
 async def _main(args):
     disable_stdout_buffering()
     ebus = create_ebus(args)
-    await load_msgdefs(ebus, args)
+    await async_load_msgdefs(ebus, args)
     value = args.value if args.value != "NONE" else None
     for msgdef in ebus.msgdefs.resolve(args.field.split(";"), filter_=lambda msgdef: msgdef.write):
-        await ebus.write(msgdef, value)
+        await ebus.async_write(msgdef, value)
