@@ -3,6 +3,7 @@ import asyncio
 import collections
 import logging
 
+from .circuitinfodecoder import decode_circuitinfos
 from .connection import CommandError, Connection
 from .const import DEFAULT_HOST, DEFAULT_PORT, DEFAULT_SCANINTERVAL, DEFAULT_SCANS, DEFAULT_TIMEOUT, OK
 from .msg import BrokenMsg, filter_msg
@@ -23,8 +24,20 @@ class Ebus:
 
     The EBUS handle, using one :any:`Connection` to an EBUSD instance.
     One EBUSD server can handle multiple :any:`Ebus` instances.
+
+    Keyword Args:
+        host (str): EBUSD host
+        port (int): EBUSD port
+        timeout (int): Connection Timeout on connect and write.
+        scaninterval (str): EBUSD scan - check interval
+        scans (str): EBUSD scan - number of intervals
+        circuitinfos (list): List with :any:`CircuitInfo`s
+        msgdefcodes (list): EBUSD Message Definition Codes
+        msgdefs (MsgDefs): Message Definitions
     """
 
+    __slots__ = ("connection", "scaninterval", "scans", "msgdefcodes", "msgdecoder", "circuitinfos")
+
     def __init__(
         self,
         host=DEFAULT_HOST,
@@ -32,13 +45,16 @@ class Ebus:
         timeout=DEFAULT_TIMEOUT,
         scaninterval=DEFAULT_SCANINTERVAL,
         scans=DEFAULT_SCANS,
+        circuitinfos=None,
+        msgdefcodes=None,
         msgdefs=None,
     ):
         self.connection = Connection(host=host, port=port, autoconnect=True, timeout=timeout)
         self.scaninterval = scaninterval
         self.scans = scans
-        self._msgdefcodes = []
+        self.msgdefcodes = msgdefcodes or []
         self.msgdecoder = MsgDecoder(msgdefs or MsgDefs())
+        self.circuitinfos = circuitinfos or []
         _LOGGER.info(f"{self}")
 
     def __repr__(self):
@@ -73,19 +89,6 @@ class Ebus:
         """Timeout."""
         return self.connection.timeout
 
-    @property
-    def msgdefcodes(self):
-        """
-        List with raw EBUSD Message Defintions Codes.
-
-        This property is writeable.
-        """
-        return self._msgdefcodes
-
-    @msgdefcodes.setter
-    def msgdefcodes(self, msgdefcodes):
-        self._msgdefcodes = msgdefcodes
-
     @property
     def msgdefs(self):
         """
@@ -105,6 +108,8 @@ class Ebus:
             timeout=self.timeout,
             scaninterval=self.scaninterval,
             scans=self.scans,
+            circuitinfos=self.circuitinfos,
+            msgdefcodes=self.msgdefcodes,
             msgdefs=self.msgdefs,
         )
 
@@ -117,6 +122,12 @@ class Ebus:
         This method waits until no new message are found.
         It checks every `scaninterval` seconds.
         The number of messages has to be stable for `scans` times.
+
+        Raises:
+            ConnectionRefusedError: If connection cannot be established
+            ConnectionError: On connection breakdown.
+            CommandError: If command failed
+            Shutdown: On EBUSD shutdown.
         """
         cnts = []
         while True:
@@ -133,12 +144,26 @@ class Ebus:
         Load Message Definitions from EBUSD.
 
         Alias for :any:`async_load_msgdefcodes` and :any:`decode_msgdefcodes`.
+
+        Raises:
+            ConnectionRefusedError: If connection cannot be established
+            ConnectionError: On connection breakdown.
+            CommandError: If command failed
+            Shutdown: On EBUSD shutdown.
         """
         await self.async_load_msgdefcodes()
         self.decode_msgdefcodes()
 
     async def async_load_msgdefcodes(self):
-        """Load EBUS Message Definition Codes from EBUSD and store to :any:`msgdefcodes`."""
+        """
+        Load EBUS Message Definition Codes from EBUSD and store to :any:`msgdefcodes`.
+
+        Raises:
+            ConnectionRefusedError: If connection cannot be established
+            ConnectionError: On connection breakdown.
+            CommandError: If command failed
+            Shutdown: On EBUSD shutdown.
+        """
         _LOGGER.info("load_msgdefcodes()")
         self.msgdefcodes = msgdefcodes = []
         await self.connection.async_request(_CMD_FINDMSGDEFS)
@@ -183,9 +208,9 @@ class Ebus:
         Raises:
             ValueError: on decoder error
             ConnectionRefusedError: If connection cannot be established
-            ConnectionError: If not connected (`autoconnect==False`)
+            ConnectionError: On connection breakdown.
             CommandError: If command failed
-            EbusdShutdown: On EBUSD shutdown.
+            Shutdown: On EBUSD shutdown.
         """
         _LOGGER.info(f"read({msgdef!r}, ttl={ttl!r})")
         return await self._async_read(msgdef, ttl)
@@ -206,13 +231,13 @@ class Ebus:
 
         Raises:
             ConnectionRefusedError: If connection cannot be established
-            ConnectionError: If not connected (`autoconnect==False`)
+            ConnectionError: On connection breakdown.
         """
         _LOGGER.info(f"write({msgdef!r}, value={value!r}, ttl={ttl!r})")
         if not msgdef.write:
             raise ValueError(f"Message is not writeable '{msgdef}'")
         fullmsgdef = self.msgdefs.get(msgdef.circuit, msgdef.name)
-        if fullmsgdef != msgdef:
+        if len(fullmsgdef.children) != len(msgdef.children):
             # Read
             if not msgdef.read:
                 raise ValueError(f"Message is not read-modify-writable '{msgdef}'")
@@ -246,9 +271,9 @@ class Ebus:
 
         Raises:
             ConnectionRefusedError: If connection cannot be established
-            ConnectionError: If not connected (`autoconnect==False`)
+            ConnectionError: On connection breakdown.
             CommandError: If command failed
-            EbusdShutdown: On EBUSD shutdown.
+            Shutdown: On EBUSD shutdown.
         """
         _LOGGER.info(f"listen(msgdefs={msgdefs!r})")
         async for msg in self._async_listen(msgdefs):
@@ -266,13 +291,13 @@ class Ebus:
 
         Raises:
             ConnectionRefusedError: If connection cannot be established
-            ConnectionError: If not connected (`autoconnect==False`)
+            ConnectionError: On connection breakdown.
             CommandError: If command failed
-            EbusdShutdown: On EBUSD shutdown.
+            Shutdown: On EBUSD shutdown.
         """
         _LOGGER.info(f"async_setprio({msgdef!r}, setprio={setprio!r})")
         msgdef = msgdef.replace(setprio=resolve_prio(msgdef, setprio or msgdef.setprio))
-        await self._async_read(msgdef)
+        return await self._async_read(msgdef)
 
     async def async_observe(self, msgdefs=None, ttl=None):
         """
@@ -290,9 +315,9 @@ class Ebus:
 
         Raises:
             ConnectionRefusedError: If connection cannot be established
-            ConnectionError: If not connected (`autoconnect==False`)
+            ConnectionError: On connection breakdown.
             CommandError: If command failed
-            EbusdShutdown: On EBUSD shutdown.
+            Shutdown: On EBUSD shutdown.
         """
         _LOGGER.info(f"observe(msgdefs={msgdefs!r}, ttl={ttl!r})")
         msgdefs = msgdefs or self.msgdefs
@@ -379,6 +404,13 @@ class Ebus:
             info[name.strip()] = value.strip()
         return info
 
+    async def async_load_circuitinfos(self):
+        """Load EBUSD Circuit Information and store in :any:`circuitinfos`."""
+        _LOGGER.info("load_circuitinfos()")
+        await self.connection.async_request("info")
+        lines = [line async for line in self.connection.async_read()]
+        self.circuitinfos = decode_circuitinfos(lines)
+
     async def async_cmd(self, cmd, infinite=False, check=False):
         """
         Send EBUS_Command_ `cmd` to EBUSD and Receive Response.
@@ -397,9 +429,9 @@ class Ebus:
 
         Raises:
             ConnectionRefusedError: If connection cannot be established
-            ConnectionError: If not connected (`autoconnect==False`)
+            ConnectionError: On connection breakdown.
             CommandError: If command failed and `check==True`.
-            EbusdShutdown: On EBUSD shutdown.
+            Shutdown: On EBUSD shutdown.
         """
         _LOGGER.info(f"cmd({cmd!r}, infinite={infinite!r}, check={check!r})")
         await self.connection.async_write(cmd)
@@ -410,7 +442,7 @@ class Ebus:
         try:
             await self.connection.async_request("read", msgdef.name, c=msgdef.circuit, p=msgdef.setprio, m=ttl)
             line = await self.connection.async_readresp(check=False)
-        except CommandError as exc:
+        except CommandError as exc:  # pragma: no cover
             return BrokenMsg(msgdef, str(exc))
         else:
             return self.msgdecoder.decode_value(msgdef, line)
@@ -432,6 +464,6 @@ class Ebus:
                 return self.msgdecoder.decode_line(line)
             except UnknownMsgError:
                 pass
-            except ValueError as exc:
+            except ValueError as exc:  # pragma: no cover
                 _LOGGER.warning(f"Cannot decode message in {line!r}: {exc}")
         return None
