@@ -52,8 +52,7 @@ class MsgDefs:
                 msgdefs[idx] = joined
                 break
         else:
-            if msgdef not in msgdefs:
-                msgdefs.append(msgdef)
+            msgdefs.append(msgdef)
 
     def get(self, circuit, name):
         """
@@ -91,6 +90,8 @@ class MsgDefs:
         Returns:
             MsgDefs: Message Definitions
         """
+        if isinstance(patterns, str):
+            patterns = patterns.split(";")
         msgdefs = MsgDefs()
         for pattern in patterns:
             for msgdef in self._resolve(pattern.strip()):
